'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = reduxForm;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _actions = require('./actions');

var _util = require('./util');

/**
 * @param sliceName The key in the state corresponding to the data in this form
 * @param validate [optional] A validation function that takes all the data and returns all the errors
 * @param asyncConfig [optional] {
 *   validate: an asynchronous validation function that takes all the data and returns a promise
 *             that resolves to async validation errors, or {} if none,
 *   fields: an array of field names for which handleBlur should trigger an async validation call
 * }
 */

function reduxForm(sliceName) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  var validate = function validate() {
    return {};
  };
  var asyncConfig = undefined;
  if (typeof args[0] === 'function') {
    validate = args.shift();
  }
  if (typeof args[0] === 'object') {
    asyncConfig = args[0];
  }
  function runValidation(form) {
    var syncErrors = validate(form.data);
    var asyncErrors = _extends({}, form.asyncErrors);
    var valid = !!(syncErrors.valid && asyncErrors.valid); // !! to convert falsy to boolean
    return _extends({}, syncErrors, asyncErrors, {
      valid: valid
    });
  }
  return function (DecoratedComponent) {
    return (function (_Component) {
      _inherits(ReduxForm, _Component);

      function ReduxForm() {
        _classCallCheck(this, ReduxForm);

        _Component.apply(this, arguments);
      }

      ReduxForm.prototype.render = function render() {
        var _props = this.props;
        // eslint-disable-line no-shadow
        var form = _props.form;
        var sliceName = _props.sliceName;
        var dispatch = _props.dispatch;

        var passableProps = _objectWithoutProperties(_props, ['form', 'sliceName', 'dispatch']);

        var handleBlur = function handleBlur(name, value) {
          return function (event) {
            dispatch(_actions.blur(sliceName, name, value || event.target.value));
            if (asyncConfig && asyncConfig.validate && asyncConfig.fields && ~asyncConfig.fields.indexOf(name)) {
              dispatch(_actions.startAsyncValidation(sliceName));
              asyncConfig.validate(form.data).then(function (asyncErrors) {
                return dispatch(_actions.stopAsyncValidation(sliceName, asyncErrors));
              });
            }
          };
        };
        var handleChange = function handleChange(name, value) {
          return function (event) {
            return dispatch(_actions.change(sliceName, name, value || event.target.value));
          };
        };
        var pristine = _util.isPristine(form.initial, form.data);

        var _runValidation = runValidation(form);

        var valid = _runValidation.valid;

        var errors = _objectWithoutProperties(_runValidation, ['valid']);

        return _react2['default'].createElement(DecoratedComponent, _extends({
          asyncValidating: form.asyncValidating,
          data: form.data,
          dirty: !pristine,
          dispatch: dispatch,
          errors: errors,
          handleBlur: handleBlur,
          handleChange: handleChange,
          initializeForm: function (data) {
            return dispatch(_actions.initialize(sliceName, data));
          },
          invalid: !valid,
          pristine: pristine,
          resetForm: function () {
            return dispatch(_actions.reset(sliceName));
          },
          touch: function () {
            for (var _len2 = arguments.length, fields = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              fields[_key2] = arguments[_key2];
            }

            return dispatch(_actions.touch.apply(undefined, [sliceName].concat(fields)));
          },
          touched: form.touched,
          touchAll: function () {
            return dispatch(_actions.touchAll(sliceName));
          },
          untouch: function () {
            for (var _len3 = arguments.length, fields = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
              fields[_key3] = arguments[_key3];
            }

            return dispatch(_actions.untouch.apply(undefined, [sliceName].concat(fields)));
          },
          untouchAll: function () {
            return dispatch(_actions.untouchAll(sliceName));
          },
          valid: valid
        }, passableProps)); // pass other props
      };

      _createClass(ReduxForm, null, [{
        key: 'displayName',
        value: 'ReduxForm(' + _util.getDisplayName(DecoratedComponent) + ')',
        enumerable: true
      }, {
        key: 'DecoratedComponent',
        value: DecoratedComponent,
        enumerable: true
      }, {
        key: 'propTypes',
        value: {
          sliceName: _react.PropTypes.string,
          form: _react.PropTypes.object.isRequired,
          dispatch: _react.PropTypes.func.isRequired
        },
        enumerable: true
      }, {
        key: 'defaultProps',
        value: {
          sliceName: sliceName
        },
        enumerable: true
      }]);

      return ReduxForm;
    })(_react.Component);
  };
}

module.exports = exports['default'];